//
//  QHVCNet+GodSees.h
//  QHVCNetKit
//
//  Created by yangkui on 2019/4/25.
//  Copyright © 2019 yangkui. All rights reserved.
//

#import "QHVCNet.h"

NS_ASSUME_NONNULL_BEGIN

#pragma mark - 外部属性定义 -

//用于设置码流，类型见QHVCNetGodSeesStreamType
extern NSString* const kQHVCNetGodSeesOptionsStreamTypeKey;

//最大重连次数，默认3次
extern NSString* const kQHVCNetGodSeesOptionsMaxReconnectCountKey;

//业务决定是否使用外部长连，值见：QHVCNetLongConnectService。默认值：QHVCNetLongConnectServiceSDK
extern NSString* const kQHVCNetGodSeesOptionsUseLongConnectKey;

//业务决定使用签名鉴权规则，0：clicentId,1:serialNumber
extern NSString* const kQHVCNetGodSeesOptionsSignRuleKey;

//设备工作模式，见QHVCNetGodSeesDeviceWorkMode定义
extern NSString* const kQHVCNetGodSeesOptionsWorkModeKey;

//如果业务使用SDK长连，相关字典返回数据的<K,V>,例如{kQHVCNetGodSeesOptionsModelKey:kQHVCNetGodSeesOptionsRtcModelKey}
extern NSString* const kQHVCNetGodSeesOptionsModelKey;
extern NSString* const kQHVCNetGodSeesOptionsModelRtcSDKKey;
extern NSString* const kQHVCNetGodSeesOptionsModelNetSDKKey;
extern NSString* const kQHVCNetGodSeesOptionsModelDeviceCommandKey;

//功耗工作模式配置参数
extern NSString* const kQHVCNetGodSeesOptionsWorkModeKey;//功耗工作模式
extern NSString* const kQHVCNetGodSeesOptionsWorkModeLongLiveValue;//长电
extern NSString* const kQHVCNetGodSeesOptionsWorkModeWakeupValue;//低功耗

//Tracker工作模式配置参数
extern NSString* const kQHVCNetGodSeesOptionsTrackerTypeKey;//Tracker工作模式
extern NSString* const kQHVCNetGodSeesOptionsP2PTrackerValue;//P2P
extern NSString* const kQHVCNetGodSeesOptionsTcpTrackerValue;//TCP
extern NSString* const kQHVCNetGodSeesOptionsP2PAndTcpTrackerValue;//P2P AND TCP

//信令工作模式配置参数
extern NSString* const kQHVCNetGodSeesOptionsSignalingEncryptModeKey;//信令加密模式
extern NSString* const kQHVCNetGodSeesSignalingEncryptValue;//信令开启加密
extern NSString* const kQHVCNetGodSeesSignalingNoEncryptValue;//信令不开启加密

//设置会话关联关系参数
extern NSString* const kQHVCNetGodSeesMutiChannelMainSidKey;//主通道的sid
extern NSString* const kQHVCNetGodSeesAuxiliaryChannelOfMultiChannelKey;//多通道辅助通道
extern NSString* const kQHVCNetGodSeesSendCommandToOldDevice;//设置relay开流信息
#pragma mark - 数据结构定义 -

/**
 以天为单位卡录时间轴数据结构
 */
@interface QHVCNetGodSeesRecordTimelineDay : NSObject

@property(nonatomic, assign) uint32_t timeS;//时间。单位：秒
@property(nonatomic, assign) uint32_t userFlag;//用户定义的数据录制类型。0：SDK内部保留，其它值由业务自定义。

@end

@interface QHVCNetGodSeesRecordSubTimeline : NSObject

@property(nonatomic, assign) uint64_t startMS;//开始时间。单位：毫秒
@property(nonatomic, assign) uint32_t durationMS;//持续时间。单位：毫秒
@property(nonatomic, assign) uint32_t userFlag;//数据录制类型。0：SDK内部保留，其它值由业务自定义。
@property(nonatomic, assign) uint64_t backgroundImage;//背景图片id，代表背景图片的时间点

@end

/**
 卡录时间轴数据结构
 */
@interface QHVCNetGodSeesRecordTimeline : NSObject

@property(nonatomic, assign) uint64_t startMS;//开始时间。单位：毫秒
@property(nonatomic, assign) uint32_t durationMS;//持续时间。单位：毫秒
@property(nonatomic, strong) NSArray<QHVCNetGodSeesRecordSubTimeline *> *subTimelines;//子timeline数组

@end

/// 指定时间点的卡录的背景的图片数据结构
@interface QHVCNetGodSeesRecordBackgroundImageData : NSObject

@property(nonatomic, assign) uint64_t timepointOfImage;//图像时间点
@property(nonatomic, assign) uint32_t imageDataLength;//图像数据长度
@property(nonatomic, strong) NSData* imageData;//图像数据
@property(nonatomic, assign) uint32_t result;//数据下载结果，0：成功，非0：失败

@end

/**
 音频流格式
 */
@interface QHVCNetGodSeesAudioStreamFormat : NSObject

@property (nonatomic, assign) int sampleRate;//采样率
@property (nonatomic, assign) int sampleBits;//采样深度
@property (nonatomic, assign) int channelNumbers;//声道数

@end

/// 下载卡录文件状态
@interface QHVCNetGodSeesDownloadRecordFileStatus : NSObject

//文件下载状态
@property (nonatomic, assign) QHVCNetGodSeesDownloadStatus downloadStatus;
//用户请求下载的卡录视频的开始时间段
@property (nonatomic, assign) unsigned long long userStartTimeMs;
//用户请求下载的卡录视频的结束时间段
@property (nonatomic, assign) unsigned long long userEndTimeMs;
//真实开始下载的时间戳，是当前接收到的卡录视频第一个视频帧的时间戳，因用户选择设置的开始、结束位置可能是空白区，因此：userStartTimeMs <= realStartTimeMs <= userEndTimeMs
@property (nonatomic, assign) unsigned long long realStartTimeMs;
//当前接收到的卡录视频的最大时间戳，realStartTimeMs <= currentDownloadTimeMs <= userEndTimeMs
@property (nonatomic, assign) unsigned long long currentDownloadTimeMs;

//SDK内部开始下载的时间戳（取的是app系统的当前时间戳），单位毫秒,文件没有下载时，值为0。文件开始下载时，取app系统的当前时间戳，且该值不再改变
@property (nonatomic, assign) unsigned long long startDownloadSystemTimeMs;
//SDK内部最后下载数据的时间戳（取的是app系统的当前时间戳），单位毫 文件没有下载时，值为0.文件开始下载且没有下载完毕时，值是app系统的当前时间戳，文件下载完毕时，该值不再改变
@property (nonatomic, assign) unsigned long long lastDownloadSystemTimeMs;
//第二错误码。不同的错误码对应的第二错误码意义不一样。
//比如：
//错误码DISHI_EC_VERIFY_TOKEN_FAILED(124)，对应的错误码有：
//     TokenCodeIndex_RelayMoreThanMax(1)，表示转发模式超过最大数量
//     TokenCodeIndex_P2pMoreThanMax(2)，表示p2p模式超过最大数量
//     TokenCodeIndex_RecordMoreThanMax(4), 表示卡录超过最大数量
//     TokenCodeIndex_UpdateFirmware_ing(5), 表示固件更新中
//     TokenCodeIndex_FormateSD_ing(6), 表示格式化卡录中
//     TokenCodeIndex_PowerOff_ing(7), 表示软关机中
@property (nonatomic, assign) unsigned int secondErrorCode;

@end

#pragma mark - 代理回调 -

@protocol QHVCNetGodSeesDelegate <NSObject>

@optional

/**
 错误监听回调

 @param sessionId 会话ID
 @param errorCode 错误码
 */
- (void)onGodSees:(NSString *)sessionId didError:(QHVCNetErrorCode)errorCode;

/**
 远端数据流拉取成功回调

 @param sessionId 会话ID
 @param status 用户token 验证结果，由业务定义值，成功：展示视频，失败：不展示视频
 @param info 附带信息
 */
- (void)onGodSees:(NSString *)sessionId didPullStreamSucceed:(NSInteger)status info:(NSString *)info;

/**
 旧设备在此回调中组装开流消息（或者直接使用该消息），并通过长连接转发给设备，进行relay开流。
 @param sessionId 会话ID
 @param info 附带信息
 */
- (void)onGodSees:(NSString *)sessionId oldDevicePlatformSendCommandToDevie:(NSString *)info;

/// 会话销毁成功回调，用户可在此回调中清除相关业务逻辑流程。
/// @param sessionId 会话ID
- (void)onGodSeesDestroySessionComplete:(NSString *)sessionId;

/// 设备主动停止用户请求
/// @param sessionId 会话ID
/// @param reasonNumber (业务自定义)
/// @param reasonMessage (业务自定义)
- (void)onGodSees:(NSString *)sessionId didDeviceStopUserRequest:(int)reasonNumber reasonMessage:(NSString *)reasonMessage;

/// 用户请求以天为单位返回指定类型卡录时间轴结果
/// @param sessionId sessionId 会话ID
/// @param data 卡录时间数据
/// @param recordType 用户指定的卡录类型
- (void)onGodSees:(NSString *)sessionId didRecordTimelineByDay:(NSArray<QHVCNetGodSeesRecordTimelineDay *> *)data recordType:(uint32_t)recordType;


/// 请求用户指定类型卡录时间轴结果
/// @param sessionId sessionId 会话ID
/// @param data 卡录时间数据
/// @param recordType 用户指定的卡录类型
- (void)onGodSees:(NSString *)sessionId didRecordTimeline:(NSArray<QHVCNetGodSeesRecordTimeline *> *)data recordType:(uint32_t)recordType;

/// 请求卡录背景图片数据回调，该方法在getGodSeesRecordBackgroundImage:timepointOfImage:调用后回调
/// @param sessionId 会话ID
/// @param data 指定时间点的卡录的背景的图片数据
- (void)onGodSees:(NSString *)sessionId didRecordBackgroundImageData:(QHVCNetGodSeesRecordBackgroundImageData *)data;

/**
 更新当前播放的时间点

 @param sessionId 会话ID
 @param timeStampByMS 当前时间戳
 */
- (void)onGodSees:(NSString *)sessionId didRecordUpdateCurrentTimeStamp:(NSUInteger)timeStampByMS;

/**
 卡录已经暂停
 
 @param sessionId 会话ID
 */
- (void)onGodSeesRecordPause:(NSString *)sessionId;

/**
 卡录已经恢复
 
 @param sessionId 会话ID
 */
- (void)onGodSeesRecordResume:(NSString *)sessionId;

/**
 卡录播放完毕

 @param sessionId 会话ID
 */
- (void)onGodSeesRecordPlayComplete:(NSString *)sessionId;

/**
 卡录seek完成

 @param sessionId 会话ID
 */
- (void)onGodSeesRecordSeekComplete:(NSString *)sessionId;

/**
 倍速播放回调
 
 @param sessionId 会话ID
 @param rate 卡录播放速度值
 */
- (void)onGodSees:(NSString *)sessionId didRecordPlaybackRate:(double)rate;

/// 卡录删除完成回调
/// @param sessionId 会话ID
/// @param errorCode 0: 成功，非0：失败，具体见错误解释
- (void)onGodSees:(NSString *)sessionId didRecordFileDeleteComplete:(QHVCNetErrorCode)errorCode;

/**
 监控网络日志信息

 @param sessionId 会话ID
 @param info 监控信息
 */
- (void)onGodSees:(NSString *)sessionId didMonitorNetworkInfo:(NSString *)info;

/**
 NetSDK将帧数据抛出，将相关数据直接吐给播放器播放。

 @param sessionId 会话ID
 @param frameDataType 数据帧类型
 @param frameData 帧数据
 @param length 帧数据长度
 @param pts 显示时间戳
 @param dts 解码时间戳
 @param isKeyFrame 是否是关键帧
 @param audioStreamFormat 音频格式
 */
-  (void)onGodSees:(NSString *)sessionId didReceiveFrameDataType:(QHVCNetGodSeesFrameDataType)frameDataType
         frameData:(const uint8_t *)frameData
      frameDataLen:(int)length
               pts:(uint64_t)pts
               dts:(uint64_t)dts
        isKeyFrame:(BOOL)isKeyFrame
 audioStreamFormat:(QHVCNetGodSeesAudioStreamFormat *)audioStreamFormat;

/**
rtc session状态

@param sessionId 会话ID
@param sessionStatus rtc会话状态
*/
- (void)onGodSees:(NSString *)sessionId rtcSessionStatus:(QHVCNetGodSeesRTCSessionStatus)sessionStatus;

/**
 借用业务信令通道发送数据回调方法
 注意：只有在设置 QHVCNetLongConnectService = QHVCNetLongConnectServiceExternal 模式下，该方法调用才有效
 
 @param data 发送的数据体
 @param destId 设备ID
 */
- (void)onGodSees:(NSString *)destId didSendSignallingData:(NSString *)data;


/**
 使用SDK信令通道发送消息回调方法
 注意：只有在设置 QHVCNetLongConnectService = QHVCNetLongConnectServiceSDK 模式下，该方法调用才有效
 
 @param data 发送的数据体
 @param destId 设备ID
 */
- (void)onGodSees:(NSString *)destId didReceiveSignallingData:(NSString *)data;


/// 卡录文件下载进度回调方法
/// @param sessionId 会话ID
/// @param fileStatusInfo 文件状态信息
- (void)onGodSees:(NSString *)sessionId didRecordFileDownloadProgress:(QHVCNetGodSeesDownloadRecordFileStatus *)fileStatusInfo;

/// 卡录文件下载完成回调方法
/// @param sessionId 会话ID
/// @param errorCode 错误码，0:成功，非0：失败
/// @param fileStatusInfo 文件状态信息
- (void)onGodSees:(NSString *)sessionId didRecordFileDownloadComplete:(QHVCNetErrorCode)errorCode fileStatusInfo:(QHVCNetGodSeesDownloadRecordFileStatus *)fileStatusInfo;

//======一下回调可废弃=======
/**
 请求卡录时间轴结果

 @param sessionId 会话ID
 @param data 卡录时间数据
 */
- (void)onGodSees:(NSString *)sessionId didRecordTimeline:(NSArray<QHVCNetGodSeesRecordTimeline *> *)data __deprecated;

/// 请求用户指定类型卡录时间轴结果(行车记录仪专用)
/// @param sessionId sessionId 会话ID
/// @param data 卡录时间数据
/// @param recordType 用户指定的卡录类型
- (void)onGodSees:(NSString *)sessionId didRecordTimelineByCar:(NSArray<QHVCNetGodSeesRecordTimeline *> *)data recordType:(uint32_t)recordType __deprecated;

@end

#pragma mark - SDK实现 -
@interface QHVCNet (GodSees)

@property (nonatomic, weak) id<QHVCNetGodSeesDelegate> godSeesDelegate;//代理回调

#pragma mark - 全局公共方法 -

/**
 启用测试环境
 
 @param testEnv YES：启用；NO：关闭
 @return 0：成功，非0:失败
 */
- (int)enableTestEnvironment:(BOOL)testEnv;

/// 是否扩展设备标识
/// 此接口专门给旧业务使用，新业务调用无效。此接口必须在- (void)startLocalServer:cacheSize:options:completion:之前调用。
/// @param enable YES: 打开 NO: 关闭。默认值是YES
- (int)enableExtendDeviceId:(BOOL)enable;

/**
 设置帝视网络连接方式
 可以是多种方式的组合，默认值: netConnectType = QHVC_NET_GODSEES_NETWORK_CONNECT_TYPE_P2P | QHVC_NET_GODSEES_NETWORK_CONNECT_TYPE_RELAY | QHVC_NET_GODSEES_NETWORK_CONNECT_TYPE_DIRECT_IP
 
 @param networkConnectType 网络连接类型
 */
- (void)setGodSeesNetworkConnectType:(QHVCNetGodSeesNetworkConnectType)networkConnectType;


/**
 设置p2p连接成功最大等待时间
 调用时机：createGodSeesSession之前调用
 
 @param timeMS 等待时间，单位：毫秒，默认值：1000毫秒
 */
- (void)setGodSeesP2PConnectionSucceedMaxWaitTime:(NSInteger)timeMS;


/**
 开启/关闭视频状态监控
 开启监控，SDK将会触发didMonitorNetworkInfo回调
 
 @param enable YES 监控,NO 不监控，默认值:NO
 */
- (void)enableGodSeesMonitorVideoState:(BOOL)enable;


/// 设置设备支持信令类型，用于兼容老设备，如果不调用，该设备长连默认值为：QHVCNetLongConnectServiceSDKTracker
/// @param serialNumber 设备唯一标识
/// @param type 长连类型
- (int)setGodSeesDeviceSupportSignalling:(NSString *)serialNumber
                                    type:(QHVCNetLongConnectService)type;

/// 设置设备sn的字符串前缀，前缀要么是该用户的：appId,要么是license的企业ID：productId
/// 用于隔离不同的企业sn，如果不调用，默认值为该企业的productId，无论是什么值，该设备的前缀必须和要访问设备的前缀保持一致
/// 必须先调用该接口后，对设备的其它操作才起作用
/// @param serialNumber 设备唯一标识
/// @param prefix 前缀标识
- (int)setGodSeesDevicePrefix:(NSString *)serialNumber
                       prefix:(NSString *)prefix;

/// 设置设备网络模式
/// @param productId 产品ID，在智汇云官网后台申请
/// @param serialNumber 设备唯一标识
/// @param mode 网络模式
- (int)setGodSeesDeviceNetworkMode:(NSString *)productId
                      serialNumber:(NSString *)serialNumber
                              mode:(QHVCNetGodSeesNetworkMode)mode;

/// 握手硬件设备
/// 如果业务使用外部长连（QHVCNetLongConnectServiceExternal），在开流前调用或者预连接前进行调用，否则不需要调用该方法。
/// @param serialNumber 设备唯一标识
- (int)handShakeHardwareDevice:(NSString *)serialNumber;

/// 设置设备扩展信息，用于提前告知设备的某些特性
/// @param serialNumber 设备唯一标识
/// @param extraInfo 扩展信息。eg: @{kQHVCNetGodSeesOptionsWorkModeKey:kQHVCNetGodSeesOptionsWorkModeLongLiveValue,//设备工作模式:低功耗/长电
///                              kQHVCNetGodSeesOptionsTrackerTypeKey:kQHVCNetGodSeesOptionsP2PAndTcpTrackerValue,//Tracker链接方式：P2P/TCP
///                              kQHVCNetGodSeesOptionsSignalingEncryptModeKey:kQHVCNetGodSeesSignalingEncryptValue}//信令安全模式：加密/不加密
- (int)setGodSeesDeviceExtraInfo:(NSString *)serialNumber
                            info:(NSDictionary *)extraInfo;

/**
 设置旧版设备平台的信息(需在preVerifySignatureByProductId、createGodSeesSessionByProductId之前调用)
 
 @param serialNumber 设备sn
 @param deviceType  设备平台类型,固定传QHVC_NET_GODSEES_DEVICE_PLATFORM_OLD
 @param dataSecretKey  从服务器获取
 @param p2pForUserKey  从服务器获取
 @return 0成功，非0表示失败
 */
- (int)setDeviceBusinessInfoBySerialNumber:(NSString *)serialNumber
                                deviceType:(QHVCNetGodSeesDevicePlatformType)deviceType
                             dataSecretKey:(NSString *)dataSecretKey
                             p2pForUserKey:(NSString *)p2pForUserKey;

/// 设置流文件保存开关
- (void)setSaveStreamFileSwitch:(BOOL)on;

/// 设置流文件保存路径,文件名
/// @param path   磁盘路径
/// @param fileName 文件名
- (void)setSaveSteamFilePath:(NSString *)path
                    fileName:(NSString *)fileName;

#pragma mark - 视频开流相关 -

/**
 设置帝视设备IP直连网络地址
 
 @param productId 产品ID，在智汇云官网后台申请
 @param serialNumber 设备唯一标识
 @param ip ip地址。值为NULL时，删除原来的ip地址
 @param port 设备侦听端口
 @return 0成功，非0表示失败
 */
- (int)setGodSeesDeviceNetworkAddress:(NSString *)productId
                         serialNumber:(NSString *)serialNumber
                                   ip:(NSString *)ip
                                 port:(NSInteger)port;


/// 预先验证产品签名信息
/// 建议在用户登录成功后立即调用，用于提高观看视频流的首屏体验
/// @param productId 产品ID，在智汇云官网后台申请
/// @param authTime 时间戳，业务请求计算productSign时的时间戳
/// @param randomNum 随机数，业务请求计算productSign时的随机数
/// @param productSign 产品签名,鉴定SDK业务的合法性，签名需要由业务服务器下发，密钥在智汇云官网获取
/// @param serialNumber 目标设备序列号
/// @param deviceChannelNumber 设备管道数量，IPC设备填1，NVR设备填设备实际支持的通道数量
/// @param completion 函数执行完毕回调
- (void)preVerifySignatureByProductId:(NSString *)productId
                             authTime:(NSUInteger)authTime
                            randomNum:(NSInteger)randomNum
                          productSign:(NSString *)productSign
                         serialNumber:(NSString *)serialNumber
                  deviceChannelNumber:(NSInteger)deviceChannelNumber
                           completion:(void(^)(QHVCNetErrorCode result))completion;


/// 帝视创建网络会话实例
/// 使用licence鉴权方案开流请使用此方法开流
/// @param productId 产品ID，在智汇云官网后台申请
/// @param authTime 时间戳，业务请求计算productSign时的时间戳
/// @param randomNum 随机数，业务请求计算productSign时的随机数
/// @param productSign 产品签名,鉴定SDK业务的合法性，签名需要由业务服务器下发，密钥在智汇云官网获取
/// @param sessionId 实例会话ID
/// @param userId 用户ID
/// @param serialNumber 设备唯一标识
/// @param deviceChannelNo 设备管道号，从索引1开始[1, ...]
/// @param sessionType 会话类型:直播/卡录/文件下载等
/// @param options 可为nil @{ kQHVCNetGodSeesOptionsStreamTypeKey:@(QHVCNetGodSeesStreamType),
///                       kQHVCNetGodSeesOptionsMaxReconnectCountKey:@(3)
///                      }
/// @param completion 函数执行完毕回调
- (void)createGodSeesSessionByProductId:(NSString *)productId
                               authTime:(NSUInteger)authTime
                              randomNum:(NSInteger)randomNum
                            productSign:(NSString *)productSign
                              sessionId:(NSString *)sessionId
                                 userId:(NSString *)userId
                           serialNumber:(NSString *)serialNumber
                        deviceChannelNo:(NSInteger)deviceChannelNo
                            sessionType:(QHVCNetGodSeesSessionType)sessionType
                                options:(NSDictionary *)options
                             completion:(void(^)(QHVCNetErrorCode result))completion;


/// 设置会话关联信息。调用时机，创建完毕会话后，在正式拉流之前调用。
/// @param sessionId 会话ID
/// @param key 关联的类型，具体的值见头文件声明。 kQHVCNetGodSeesMutiChannelMainSidKey:主通道的sid，value传待关联的sid，用于在同一个通道中同时传输多路码流的场景，比如：双摄像头。
/// kQHVCNetGodSeesSendCommandToOldDevice:设置relay开流信息
/// @param value 待关联值
- (void)setSessionRelevanceInfo:(NSString *)sessionId
                            key:(NSString *)key
                          value:(NSString *)value;


/// 销毁帝视网络会话实例
/// 该方法是异步操作，SDK将会触发onGodSeesDestroySessionComplet回调，然后，业务可以回调中进行一些列后续流程操作。
/// @param sessionId 实例会话ID
/// @param serialNumber 设备唯一标识
- (int)destroyGodSeesSession:(NSString *)sessionId
                serialNumber:(NSString *)serialNumber;


/**
 获取帝视指定会话的播放链接
 
 @param sessionId 实例会话ID
 @return 成功:对应的播放链接；不成功:nil
 */
- (NSString *)getGodSeesPlayUrl:(NSString *)sessionId;


/**
 拉取远端设备数据流
 该方法是异步操作，SDK将会触发didPullStreamSucceed回调，然后，业务才可以进行一系列操作设置。若验证失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
 
 @param sessionId 实例会话ID
 @param userToken 用户身份鉴权验证信息，字符串长度不能超过500个字符
 @return 0成功，非0表示失败
 */
- (int)pullRemoteDeviceDataStream:(NSString *)sessionId
                        userToken:(NSString *)userToken;

/// 播放器准备完毕，通知服务器开始接收媒体数据
/// @param sessionId 实例会话ID
/// @param enable 开启/关闭 接收数据    YES：开启接收，NO：关闭接收
- (int)notifyServerReceiveMediaData:(NSString *)sessionId enable:(BOOL)enable;

/// RTC通话时设置发送给设备的音频参数，该SDK在开启RTC通话时调用。
/// @param sessionId 实例会话ID
/// @param codecType 音频编码类型
/// @param sampleRate 音频采样率
/// @param sampleDepth 位深
/// @param audioChannel 音频声道
- (int)setSendAudioParam:(NSString *)sessionId
               codecType:(enum QHVCNetGodSeesFrameDataType)codecType
              sampleRate:(NSInteger)sampleRate
             sampleDepth:(NSInteger)sampleDepth
            audioChannel:(NSInteger)audioChannel;


/*
 **************************************************************************************************************
 * 特别注意：下面的方法一定要等设备授权通过后才能调用。
 * 即调用pullRemoteDeviceDataStream函数通知设备验证userToken成功，待回调didPullStreamSucceed执行后，才能调用下面的接口
 **************************************************************************************************************
 */

/// 开启RTC会话
/// @param sessionId 实例会话ID
/// @param mode 双工通话模式
- (int)startRTCConversation:(NSString *)sessionId mode:(QHVCNetGodSeesDeviceDuplexMode)mode;

/// 结束RTC会话
/// @param sessionId 实例会话ID
- (int)stopRTCConversation:(NSString *)sessionId;

/// 接收外部音频
/// 当开启语音对讲时，需要用户把音频数据传进来，用于固件播放
/// @param sid 实例句柄
/// @param data 音频数据
/// @param length 音频数据
/// @param ts 音频时间戳，单位毫秒
- (int)sendAudioData:(const char *)sid data:(const uint8_t *)data length:(int)length timestamp:(uint64_t)ts;

#pragma mark - 卡录相关 -

/// 以天为单位获取多种类型的卡录时间轴信息
/// 该方法是异步操作，SDK将会触发didRecordTimelineByDay:recordType:回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
/// @param sessionId 实例会话ID
/// @param recordType 卡录类型，类型由业务自定义，从1开始，可以是多种类型的组合（1|2|4），0由SDK内部保留，表示获取所有类型。
- (int)getGodSeesRecordTimelineByDays:(NSString *)sessionId recordType:(uint32_t)recordType;


/// 获取多种类型的卡录时间轴信息
/// 该方法是异步操作，SDK将会触发didRecordTimeline:recordType:回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
/// 如果要获取全部卡录信息，srartTimeMS:0,  endTimeMS:-1
/// @param sessionId 实例会话ID
/// @param recordType 卡录类型，类型由业务自定义，从1开始，可以是多种类型的组合（1|2|4），0由SDK内部保留，表示获取所有类型。
/// @param startTimeMS 希望获取时间轴的起始时间点，默认值是0
/// @param endTimeMS 希望获取时间轴的起始时间点，默认值是-1
- (int)getGodSeesRecordTimeline:(NSString *)sessionId
                     recordType:(uint32_t)recordType
                      startTime:(NSUInteger)startTimeMS
                        endTime:(NSUInteger)endTimeMS;

/// 同时获取多种类型的卡录时间轴信息，该方法支持分页请求
/// 该方法是异步操作，SDK将会触发didRecordTimeline:recordType:回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
/// 如果要获取全部卡录信息，srartTimeMS:0,  endTimeMS:-1
/// @param sessionId 实例会话ID
/// @param recordType 卡录类型，类型由业务自定义，从1开始，可以是多种类型的组合（1|2|4），0表示获取所有类型
/// @param startTimeMS 希望获取时间轴的起始时间点，默认值是0
/// @param endTimeMS 希望获取时间轴的起始时间点，默认值是-1
/// @param limitNum 分页获取时，每页包含记录数量。如果不是分页获取，该变量传0
- (int)getGodSeesRecordTimeline:(NSString *)sessionId
                     recordType:(uint32_t)recordType
                      startTime:(NSUInteger)startTimeMS
                        endTime:(NSUInteger)endTimeMS
                       limitNum:(NSUInteger)limitNum;

/// 获取卡录背景图片数据
/// SDK将会触发onGodSees:didRecordBackgroundImageData:回调
/// @param sessionId 实例会话ID
/// @param timepointOfImage 图像对应的时间点
- (int)getGodSeesRecordBackgroundImage:(NSString *)sessionId timepointOfImage:(uint64_t)timepointOfImage;

/**
 观看指定区间[position, end]的卡录内容，position是起始点，end是结束点。
 该方法是异步操作，SDK将会触发onGodSeesRecordSeekComplete回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
 当指定区间内的卡录内容播放完毕后，SDK将会触发onGodSeesRecordPlayComplete回调，业务需要在该回调中处理后续逻辑操作。
 
 @param sessionId 实例会话ID
 @param positionTimestampByMs 指定时间点（单位：毫秒）
 @param endTimestampByMs 结束时间点（单位：毫秒）如果要播放之后所有内容，传-1
 @return 0成功，非0表示失败
 */
- (int)setGodSeesRecordSeek:(NSString *)sessionId
                   position:(NSUInteger)positionTimestampByMs
                        end:(NSUInteger)endTimestampByMs;


/**
 暂停观看卡录
 该方法是异步操作，SDK将会触发onGodSeesRecordPause回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
 
 @param sessionId 实例会话ID
 @return 0成功，非0表示失败
 */
- (int)setGodSeesRecordPause:(NSString *)sessionId;


/**
 恢复观看卡录
 该方法是异步操作，SDK将会触发onGodSeesRecordResume回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
 
 @param sessionId 实例会话ID
 @return 0成功，非0表示失败
 */
- (int)setGodSeesRecordResume:(NSString *)sessionId;


/**
 设置观看卡录速度
 该方法是异步操作，SDK将会触发didRecordPlaybackRate回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
 
 @param sessionId 实例会话ID
 @param rate 速度倍数，默认:1倍速
 @return 0成功，非0表示失败
 */
- (int)setGodSeesRecordPlayRate:(NSString *)sessionId
                           rate:(double)rate;

/// 删除指定时间段内的卡录
/// 该方法是异步操作，SDK将会触发didRecordFileDeleteComplete回调，然后，业务才可以进行一系列操作设置。
/// 如果要删除的多个时间段，在时间轴上是连续的，可以只指定第一个时间段的开始时间点，和最后一个时间段的结尾时间点。
/// @param sessionId 实例会话ID
/// @param timelineData 将要被删除的时间段数组
/// @return 0成功，非0表示失败
- (int)deleteGodSeesRecordFile:(NSString *)sessionId
                  timelineData:(NSArray<QHVCNetGodSeesRecordTimeline *> *)timelineData;

#pragma mark - 信令相关 -

/**
 帝视接收外部信令通道数据
 注意：只有在设置 QHVCNetLongConnectService = QHVCNetLongConnectServiceExternal 模式下，该方法调用才有效
 相关回调：onGodSees:didSendSignallingData:
 
 @param data 信令数据
 @return 0成功，非0表示失败
 */
- (int)receiveGodSeesSignallingData:(NSString *)data;

/**
 利用SDK内部通道发送信令消息
 注意：只有在设置 QHVCNetLongConnectService = QHVCNetLongConnectServiceSDK 模式下，该方法调用才有效
 相关回调：onGodSees:didReceiveSignallingData:
 
 @param data 信令数据
 @param destId 设备ID
 @return 0成功，非0表示失败
 */
- (int)sendSignallingData:(NSString *)data toDestId:(NSString *)destId;

#pragma mark - 安全相关 -

/**
 更新帝视视频流解密秘钥
 
 @param keys 密钥表，形如 [{"27":"sfee23"},{"28":"fsjei"},@{"29":"ejis"}]
 @param serialNumber 设备唯一标识
 @param durationBySecond 密钥更新周期,单位：秒
 @return 0成功，非0表示失败
 */
- (int)updateGodSeesVideoStreamSecurityKeys:(NSArray<NSDictionary<NSString *, NSString *> *> *)keys
                               serialNumber:(NSString *)serialNumber
                                   duration:(NSInteger)durationBySecond;

#pragma mark - 文件下载 -

/// 设置下载文件状态回调间隔，全局函数，可以在任意时刻调用
/// @param intervalMs 回调间隔，最小100 ms
- (int)setGodSessRecordFileDownloadStatusCallbackInterval:(int)intervalMs;

/// 以licence鉴权方式进行请求获取设备文件下载的链接
/// @param productId 产品ID，在智汇云官网后台申请
/// @param authTime 时间戳，业务请求计算productSign时的时间戳
/// @param randomNum 随机数，业务请求计算productSign时的随机数
/// @param productSign 产品签名,鉴定SDK业务的合法性，签名需要由业务服务器下发，密钥在智汇云官网获取
/// @param userId 用户ID
/// @param serialNumber 设备唯一标识
/// @param token 待设备验证的标识，该标识由业务服务器验证合法性，字符串长度不能超过63个字符
/// @param rangeStart 请求文件内容的起始位置，与http的range语义相同
/// @param rangeEnd 请求文件内容的结束位置，与http的range语义相同
/// @param maxReconnectCount 最大重连次数，-1：无限重连。
/// @param completion 获取的下载链接回调，下载地址通过resultUrlString返回
- (void)getGodSeesDeviceFileDownloadUrlByProductId:(NSString *)productId
                                          authTime:(NSUInteger)authTime
                                         randomNum:(NSInteger)randomNum
                                       productSign:(NSString *)productSign
                                           fileKey:(NSString *)fileKey
                                            userId:(NSString *)userId
                                      serialNumber:(NSString *)serialNumber
                                             token:(NSString *)token
                                        rangeStart:(NSUInteger)rangeStart
                                          rangeEnd:(NSUInteger)rangeEnd
                                 maxReconnectCount:(NSInteger)maxReconnectCount
                                        completion:(void(^)(BOOL result, NSString *resultUrlString, NSError* error))completion;

/// 以licence鉴权方式进行请求获取设备的卡录数据，并保存到指定的文件destMp4FileName。此接口暂时不支持断点续传。
/// 创建下载任务成功后，需调用startGodSeesDownloadRecordFile启动任务下载。
/// 当前正在下载的任务会回调：onGodSees:didRecordFileDownloadProgress方法，
/// 下载完毕会回调：onGodSees: didRecordFileDownloadComplete: fileStatusInfo:方法获取设备的卡录数据，并保存到指定的文件dest_mp4_file_name。
/// 需多通道下载时，需调用- (void)setSessionRelevanceInfo:(NSString *)sessionId key:(NSString *)key value:(NSString *)value;接口进行关联多个通道。例如setSessionRelevanceInfo:sid key:kQHVCNetGodSeesAuxiliaryChannelOfMultiChannelKey value:@"2,1";表示下载第2通道通道的主码流。下载完毕后，业务需自行合并文件。
/// @param productId  产品ID，在智汇云官网后台申请
/// @param authTime 时间戳，业务请求计算productSign时的时间戳
/// @param randomNum 随机数，业务请求计算productSign时的随机数
/// @param productSign 产品签名,鉴定SDK业务的合法性，签名需要由业务服务器下发，密钥在智汇云官网获取
/// @param sessionId 实例会话ID
/// @param userId 用户ID
/// @param userToken 待设备验证的标识，该标识由业务服务器验证合法性，字符串长度不能超过512个字符
/// @param serialNumber 设备唯一标识
/// @param deviceChannelNo 设备管道号，从索引1开始[1, ...]
/// @param startTimeMS 卡录视频的起始时间戳
/// @param endTimeMS 卡录视频的结束时间戳，注意：startTimeMS < endTimeMS
/// @param destMp4FileName 保存到目标mp4文件，必须包含全路径,例如：xxx/aa.mp4
/// @param options 可为nil @{ kQHVCNetGodSeesOptionsStreamTypeKey:@(QHVCNetGodSeesStreamType),
///                       kQHVCNetGodSeesOptionsMaxReconnectCountKey:@(3)
///                      }
/// @param completion 函数执行完毕回调
- (void)createGodSeesDownloadRecordFileByProductId:(NSString *)productId
                                          authTime:(NSUInteger)authTime
                                         randomNum:(NSInteger)randomNum
                                       productSign:(NSString *)productSign
                                         sessionId:(NSString *)sessionId
                                            userId:(NSString *)userId
                                         userToken:(NSString *)userToken
                                      serialNumber:(NSString *)serialNumber
                                   deviceChannelNo:(NSInteger)deviceChannelNo
                                         startTime:(unsigned long long)startTimeMS
                                           endTime:(unsigned long long)endTimeMS
                                   destMp4FileName:(NSString *)destMp4FileName
                                           options:(NSDictionary *)options
                                        completion:(void(^)(QHVCNetErrorCode result))completion;

/// 开始下载卡录文件
/// @param sessionId 实例会话ID
- (int)startGodSeesDownloadRecordFile:(NSString *)sessionId;

/// 强制停止下载卡录文件，结束本次会话
/// @param sessionId 实例会话ID
/// @param deleteMp4File 是否删除已经保存的mp4文件。YES： 删除  NO：保留
- (int)stopGodSeesDownloadRecordFile:(NSString *)sessionId
                deleteMp4File:(BOOL)deleteMp4File;

/// 暂停下载卡录文件
/// @param sessionId 实例会话ID
- (int)pauseGodSeesDownloadRecordFile:(NSString *)sessionId;

/// 恢复下载卡录文件
/// @param sessionId 实例会话ID
- (int)resumeGodSeesDowloadRecordFile:(NSString *)sessionId;

#pragma mark - 【可弃用方法】 -

/**
 预连接设备，为每个通道提前建立连接
 用于提前建立网络通道，节省用户拉取视频等待时间，提高用户体验
 
 @param serialNumber 需要预连的设备Id
 @param deviceChannelNumber 设备通道数量，如果一个NVR设备有64个通道，实际同时观看16个通道，就传16
 @return 0成功，非0表示失败
 */
- (int)preConnectGodSeesDevice:(NSString *)serialNumber
           deviceChannelNumber:(NSInteger)deviceChannelNumber __deprecated;

/**
 帝视创建网络会话实例
 使用非licence鉴权方案开流请使用此方法开流
 
 @param sessionId 实例会话ID
 @param clientId 客户端ID
 @param serviceName  服务标识，在智汇云官网后台申请
 @param userId 用户ID
 @param serialNumber 设备唯一标识
 @param deviceChannelNo 设备管道号，从索引1开始[1, ...]
 @param serviceSign 业务签名,鉴定SDK业务的合法性，签名需要由业务服务器下发，密钥在智汇云官网获取
 @param sessionType 会话类型:直播/卡录/文件下载等
 @param options 可为nil @{@"kQHVCNetGodSeesOptionsStreamTypeKey":@(QHVCNetGodSeesStreamType)
                        }
 @param completion 函数执行完毕回调
 */
- (void)createGodSeesSession:(NSString *)sessionId
                    clientId:(NSString *)clientId
                 serviceName:(NSString *)serviceName
                      userId:(NSString *)userId
                serialNumber:(NSString *)serialNumber
             deviceChannelNo:(NSInteger)deviceChannelNo
                 serviceSign:(NSString *)serviceSign
                 sessionType:(QHVCNetGodSeesSessionType)sessionType
                     options:(NSDictionary *)options
                  completion:(void(^)(QHVCNetErrorCode result))completion __deprecated;

/**
 获取卡录时间轴信息
 该方法是异步操作，SDK将会触发didRecordTimeline:回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
 如果要获取全部卡录信息，srartTimeMS:0,  endTimeMS:-1
 
 @param sessionId 实例会话ID
 @param srartTimeMS 卡录开始时间
 @param endTimeMS 卡录结束时间
 @return 0成功，非0表示失败
 */
- (int)getGodSeesRecordTimeline:(NSString *)sessionId
                      startTime:(NSUInteger)srartTimeMS
                        endTime:(NSUInteger)endTimeMS __deprecated;

/// 获取指定类型卡录时间轴信息(记录仪专用)
/// 该方法是异步操作，SDK将会触发didRecordTimeline:recordType:回调，然后，业务才可以进行一系列操作设置。若加载失败，SDK将会触发didError回调，请根据相关错误码进行逻辑处理。
/// 如果要获取全部卡录信息，srartTimeMS:0,  endTimeMS:-1
/// @param sessionId 实例会话ID
/// @param recordType 卡录类型。保留值：0,客户不能使用，其余的值客户可自定义
/// @param srartTimeMS 卡录开始时间
/// @param endTimeMS 卡录结束时间
- (int)getGodSeesRecordTimelineByCar:(NSString *)sessionId
                          recordType:(int)recordType
                           startTime:(NSUInteger)srartTimeMS
                             endTime:(NSUInteger)endTimeMS __deprecated;

/**
 获取设备文件下载的链接
 
 @param fileKey 指定设备上的文件的标识符
 @param clientId 客户端ID
 @param serviceName 服务标识，在帝视官网后台申请
 @param userId 用户ID
 @param serialNumber 设备唯一标识
 @param serviceSign 服务签名,帝视鉴定SDK业务的合法性，签名需要由业务服务器下发，密钥在帝视官网获取
 @param token 待设备验证的标识，该标识由业务服务器验证合法性，字符串长度不能超过63个字符
 @param rangeStart 请求文件内容的起始位置，与http的range语义相同
 @param rangeEnd 请求文件内容的结束位置，与http的range语义相同
 @param maxReconnectCount 最大重连次数，-1：无限重连。
 @param completion 获取的下载链接回调，下载地址通过resultUrlString返回
 */
- (void)getDeviceFileDownloadUrlWithFileKey:(NSString *)fileKey
                                   clientId:(NSString *)clientId
                                serviceName:(NSString *)serviceName
                                     userId:(NSString *)userId
                               serialNumber:(NSString *)serialNumber
                                serviceSign:(NSString *)serviceSign
                                      token:(NSString *)token
                                 rangeStart:(NSUInteger)rangeStart
                                   rangeEnd:(NSUInteger)rangeEnd
                          maxReconnectCount:(NSInteger)maxReconnectCount
                                 completion:(void(^)(BOOL result, NSString *resultUrlString, NSError* error))completion __deprecated;

@end

NS_ASSUME_NONNULL_END
